"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createPageElementsCrud = void 0;
var _handlerGraphql = require("@webiny/handler-graphql");
var _error = _interopRequireDefault(require("@webiny/error"));
var _pubsub = require("@webiny/pubsub");
var _validation = require("./pageElements/validation");
var _utils = require("@webiny/utils");
const createPageElementsCrud = params => {
  const {
    context,
    storageOperations,
    pagesPermissions,
    getLocaleCode,
    getTenantId
  } = params;

  // create
  const onPageElementBeforeCreate = (0, _pubsub.createTopic)("pageBuilder.onPageElementBeforeCreate");
  const onPageElementAfterCreate = (0, _pubsub.createTopic)("pageBuilder.onPageElementAfterCreate");
  // update
  const onPageElementBeforeUpdate = (0, _pubsub.createTopic)("pageBuilder.onPageElementBeforeUpdate");
  const onPageElementAfterUpdate = (0, _pubsub.createTopic)("pageBuilder.onPageElementAfterUpdate");
  // delete
  const onPageElementBeforeDelete = (0, _pubsub.createTopic)("pageBuilder.onPageElementBeforeDelete");
  const onPageElementAfterDelete = (0, _pubsub.createTopic)("pageBuilder.onPageElementAfterDelete");
  return {
    /**
     * Introduced in 5.34.0
     */
    onPageElementBeforeCreate,
    onPageElementAfterCreate,
    onPageElementBeforeUpdate,
    onPageElementAfterUpdate,
    onPageElementBeforeDelete,
    onPageElementAfterDelete,
    async getPageElement(id) {
      await pagesPermissions.ensure({
        rwd: "r"
      });
      const params = {
        where: {
          tenant: getTenantId(),
          locale: getLocaleCode(),
          id
        }
      };
      let pageElement = null;
      try {
        pageElement = await storageOperations.pageElements.get(params);
        if (!pageElement) {
          return null;
        }
      } catch (ex) {
        throw new _error.default(ex.message || "Could not get pageElement by slug.", ex.code || "GET_PAGE_ELEMENT_ERROR", {
          ...(ex.data || {}),
          params
        });
      }
      await pagesPermissions.ensure({
        owns: pageElement.createdBy
      });
      return pageElement;
    },
    async listPageElements(params) {
      await pagesPermissions.ensure({
        rwd: "r"
      });
      const {
        sort
      } = params || {};
      const listParams = {
        where: {
          tenant: getTenantId(),
          locale: getLocaleCode()
        },
        limit: 1000,
        sort: Array.isArray(sort) && sort.length > 0 ? sort : ["createdOn_ASC"]
      };

      // If user can only manage own records, let's add that to the listing.
      if (await pagesPermissions.canAccessOnlyOwnRecords()) {
        const identity = context.security.getIdentity();
        listParams.where.createdBy = identity.id;
      }
      try {
        const [items] = await storageOperations.pageElements.list(listParams);
        return items;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not list all page elements.", ex.code || "LIST_PAGE_ELEMENTS_ERROR", {
          params
        });
      }
    },
    async createPageElement(input) {
      await pagesPermissions.ensure({
        rwd: "w"
      });
      const validation = await (0, _validation.createPageElementsCreateValidation)().safeParseAsync(input);
      if (!validation.success) {
        throw (0, _utils.createZodError)(validation.error);
      }
      const id = (0, _utils.mdbid)();
      const identity = context.security.getIdentity();
      const pageElement = {
        ...validation.data,
        tenant: getTenantId(),
        locale: getLocaleCode(),
        id,
        createdOn: new Date().toISOString(),
        createdBy: {
          id: identity.id,
          type: identity.type,
          displayName: identity.displayName
        }
      };
      try {
        await onPageElementBeforeCreate.publish({
          pageElement
        });
        const result = await storageOperations.pageElements.create({
          input: validation.data,
          pageElement
        });
        await onPageElementAfterCreate.publish({
          pageElement
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not create page element.", ex.code || "CREATE_PAGE_ELEMENT_ERROR", {
          ...(ex.data || {}),
          pageElement
        });
      }
    },
    async updatePageElement(id, input) {
      await pagesPermissions.ensure({
        rwd: "w"
      });
      const original = await this.getPageElement(id);
      if (!original) {
        throw new _handlerGraphql.NotFoundError(`Page element "${id}" not found.`);
      }
      await pagesPermissions.ensure({
        owns: original.createdBy
      });
      const validation = await (0, _validation.createPageElementsUpdateValidation)().safeParseAsync(input);
      if (!validation.success) {
        throw (0, _utils.createZodError)(validation.error);
      }
      const data = (0, _utils.removeUndefinedValues)(validation.data);
      const pageElement = {
        ...original,
        ...data
      };
      try {
        await onPageElementBeforeUpdate.publish({
          original,
          pageElement
        });
        const result = await storageOperations.pageElements.update({
          input: data,
          original,
          pageElement
        });
        await onPageElementAfterUpdate.publish({
          original,
          pageElement: result
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not update page element.", ex.code || "UPDATE_PAGE_ELEMENT_ERROR", {
          ...(ex.data || {}),
          original,
          pageElement
        });
      }
    },
    async deletePageElement(slug) {
      await pagesPermissions.ensure({
        rwd: "d"
      });
      const pageElement = await this.getPageElement(slug);
      if (!pageElement) {
        throw new _handlerGraphql.NotFoundError(`PageElement "${slug}" not found.`);
      }
      await pagesPermissions.ensure({
        owns: pageElement.createdBy
      });
      try {
        await onPageElementBeforeDelete.publish({
          pageElement
        });
        await storageOperations.pageElements.delete({
          pageElement
        });
        await onPageElementAfterDelete.publish({
          pageElement
        });
      } catch (ex) {
        throw new _error.default(ex.message || "Could not delete page element.", ex.code || "DELETE_PAGE_ELEMENT_ERROR", {
          ...(ex.data || {}),
          pageElement
        });
      }
    }
  };
};
exports.createPageElementsCrud = createPageElementsCrud;

//# sourceMappingURL=pageElements.crud.js.map